@@ -2,10 +2,8 @@
     function (object, alpha = 0.05, beta = 0.01, step = 100, perm.max = 9999, 
               by = NULL, ...) 
 {
-    if (is.null(object$CA))
-        stop("Impossible analysis: no residual unconstrained component")
-    if (is.null(object$CCA))
-        stop("Nothing to analyse: no constrained component")
+    if (is.null(object$CA) || is.null(object$CCA))
+        return(anova.ccanull(object))
     perm.max <- max(step-1, perm.max)
     if (perm.max %% step == 0)
         perm.max <- perm.max - 1
