@@ -1,10 +1,9 @@
-
 ###
 ### Editable Tcl/Tk plot for ordination
 ###
 `orditkplot` <-
     function(x, display = "species", choices = 1:2, width, xlim, ylim,
-             tcex=0.8, pcol, pbg, pcex = 0.7,
+             tcex=0.8, tcol, pch = 1, pcol, pbg, pcex = 0.7,
              labels,  ...)
 {
     if (!capabilities("tcltk"))
@@ -21,12 +20,19 @@
                  "family", "fg", "font", "font.axis", "font.lab", "lheight",
                  "lwd", "mar", "mex", "mgp", "ps", "tcl", "las")
     ## Get par given in the command line and put them to p
-    dots <- match.call(expand.dots = FALSE)$...
+    if (inherits(x, "orditkplot")) {
+        dots <- x$par
+        for (arg in names(x$args))
+            assign(arg, unlist(x$args[arg]))
+    } else {
+        dots <- match.call(expand.dots = FALSE)$...
+    }
     if (length(dots) > 0) {
         dots <- dots[names(dots) %in% sparnam]
         ## eval() or mar=c(4,4,1,1) will be a call, not numeric
         dots <- lapply(dots, function(x) if (is.call(x)) eval(x) else x)
-        p <- check.options(new = dots, name.opt = "p", envir = environment())
+        p <- check.options(new = dots, name.opt = "p",
+                           envir = environment())
     }
     savepar <- p[sparnam]
     PPI <- 72                                         # Points per Inch
@@ -36,12 +42,10 @@
     diam <- round(pcex * DIAM * p2p, 1)
     ## Sanitize colours
     sanecol <- function(x) {
-        if (is.na(x))
-            x <- ""
-        else if (is.numeric(x))
+        if (is.numeric(x))
             x <- palette()[x]
-        else if (x == "transparent")
-            x <- ""
+        x <- gsub("transparent", "", x)
+        x[is.na(x)] <- ""
         x
     } 
     p$bg <- sanecol(p$bg)
@@ -49,13 +53,16 @@
     p$col <- sanecol(p$col)
     p$col.axis <- sanecol(p$col.axis)
     p$col.lab <- sanecol(p$col.lab)
-    ## Point colours
+    ## Point and label colours
     if (missing(pcol))
         pcol <- p$col
     if (missing(pbg))
         pbg <- "transparent"
+    if (missing(tcol))
+        tcol <- p$col
     pcol <- sanecol(pcol)
     pbg <- sanecol(pbg)
+    tcol <- sanecol(tcol)
     ## Define fonts
     idx <- match(p$family, c("","serif","sans","mono"))
     if (!is.na(idx))
@@ -63,9 +70,77 @@
     saneslant <- function(x) {
         list("roman", "bold", "italic", c("bold", "italic"))[[x]]
     }
-    fnt <- c(p$family, round(p$ps*p$cex*tcex), saneslant(p$font))
+    ## fnt must be done later, since family, font and size can be
+    ## vectors and slant can be of length 1 or 2
+    ## fnt <- c(p$family, round(p$ps*p$cex*tcex), saneslant(p$font))
+    labfam <- p$family
+    labsize <- round(p$ps * p$cex * tcex)
     fnt.axis <- c(p$family, round(p$ps*p$cex.axis), saneslant(p$font.axis))
     fnt.lab <- c(p$family, round(p$ps*p$cex.lab), saneslant(p$font.lab))
+    ## Imitate R plotting symbols pch
+    SQ <- sqrt(2)     # Scaling factor for plot
+    Point <- function(x, y, pch, col, fill, diam) {
+        x <- round(x)
+        y <- round(y)
+        switch(as.character(pch),
+               "0" = Point(x, y, 22, col, fill = "", diam),
+               "1" = Point(x, y, 21, col, fill = "", diam),
+               "2" = Point(x, y, 24, col, fill = "", diam),
+               "3" = {tkcreate(can, "line",
+                               x, y+SQ*diam, x, y-SQ*diam, fill=col)
+                      tkcreate(can, "line",
+                               x+SQ*diam, y, x-SQ*diam, y, fill=col)},
+               "4" = {tkcreate(can, "line",
+                               x-diam, y-diam, x+diam, y+diam, fill=col)
+                      tkcreate(can, "line",
+                               x-diam, y+diam, x+diam, y-diam, fill=col)},
+               "5" = Point(x, y, 23, col, fill = "", diam),
+               "6" = Point(x, y, 25, col, fill = "", diam),
+               "7" = {Point(x, y, 4, col, fill, diam)
+                      Point(x, y, 0, col, fill, diam)},
+               "8" = {Point(x, y, 3, col, fill, diam)
+                      Point(x, y, 4, col, fill, diam)},
+               "9" = {Point(x, y, 3, col, fill, diam)
+                      Point(x, y, 5, col, fill, diam)},
+               "10" = {Point(x, y, 3, col, fill, diam/SQ)
+                       Point(x, y, 1, col, fill, diam)},
+               "11" = {Point(x, y, 2, col, fill, diam)
+                       Point(x, y, 6, col, fill, diam)},
+               "12" = {Point(x, y, 3, col, fill, diam/SQ)
+                       Point(x, y, 0, col, fill, diam)},
+               "13" = {Point(x, y, 4, col, fill, diam)
+                       Point(x, y, 1, col, fill, diam)},
+               "14" = {tkcreate(can, "line", x-diam, y-diam, x, y+diam,
+                                fill = col)
+                       tkcreate(can, "line", x+diam, y-diam, x, y+diam,
+                                fill = col)
+                       Point(x, y, 0, col, fill, diam)},
+               "15" = Point(x, y, 22, col = col, fill = col, diam),
+               "16" = Point(x, y, 21, col = col, fill = col, diam),
+               "17" = Point(x, y, 24, col = col, fill = col, diam),
+               "18" = Point(x, y, 23, col = col, fill = col, diam/SQ),
+               "19" = Point(x, y, 21, col = col, fill = col, diam),
+               "20" = Point(x, y, 21, col = col, fill = col, diam/2),
+               "21" = tkcreate(can, "oval", x-diam, y-diam,
+               x+diam, y+diam, outline = col, fill = fill),
+               "22" = tkcreate(can, "rectangle", x-diam, y-diam,
+               x+diam, y+diam, outline = col, fill = fill),
+               "23" = tkcreate(can, "polygon", x, y+SQ*diam,
+               x+SQ*diam, y, x, y-SQ*diam, x-SQ*diam, y,
+               outline = col, fill = fill),
+               "24" = tkcreate(can, "polygon", x, y-SQ*diam,
+               x+sqrt(6)/2*diam, y+SQ/2*diam, x-sqrt(6)/2*diam, y+SQ/2*diam,
+               outline = col, fill = fill),
+               "25" = tkcreate(can, "polygon", x, y+SQ*diam,
+               x+sqrt(6)/2*diam, y-SQ/2*diam, x-sqrt(6)/2*diam, y-SQ/2*diam,
+               outline = col, fill = fill),
+               "o" = Point(x, y, 1, col, fill, diam),
+               ## default: text with dummy location of the label
+               {tkcreate(can, "text",
+                        x, y, text = as.character(pch), fill = col)
+                Point(x, y, 21, col="", fill="", diam)}
+                )
+    }
 
 ############################
 ### Initialize Tcl/Tk Window
@@ -99,12 +174,19 @@
             xy[as.numeric(tclvalue(id[[nm]])),] <- xy2usr(nm)
         }
         curdim <- round(c(width, height) /PPI/p2p, 2)
-        if (pbg == "")
-            pbg <- "transparent"
-        if (pcol == "")
-            pcol <- "transparent"
-        args <- list(tcex = tcex, pcol = pcol, pbg = pbg, pcex = pcex,
-                     xlim = xlim, ylim = ylim)
+        ## Sanitize colours for R plot
+        pbg[pbg == ""]  <- "transparent"
+        pcol[pcol == ""] <- "transparent"
+        ## Reduce vector args if all entries are constant
+        argcollapse <- function(x)
+            if (length(unique(x)) == 1) x[1] else x
+        pch <- argcollapse(pch)
+        pcol <- argcollapse(pcol)
+        pbg <- argcollapse(pbg)
+        tcol <- argcollapse(tcol)
+        ## Save
+        args <- list(tcex = tcex, tcol = tcol, pch = pch, pcol = pcol,
+                     pbg = pbg, pcex = pcex, xlim = xlim, ylim = ylim)
         xy <- list(labels = xy, points = sco, par = savepar, args = args,
                    dim = curdim)
         class(xy) <- "orditkplot"
@@ -202,11 +284,35 @@
     }
     if (!missing(labels))
         rownames(sco) <- labels
+    ## Recycle graphical parameters in plots
+    nr <- nrow(sco)
+    pcol <- rep(pcol, length=nr)
+    pbg <- rep(pbg, length=nr)
+    pch <- rep(pch, length=nr)
+    tcol <- rep(tcol, length=nr)
+    diam <- rep(diam, length=nr)
+    labfam <- rep(labfam, length=nr)
+    labsize <- rep(labsize, length=nr)
+    if (inherits(x, "ordipointlabel"))
+        labfnt <- attr(x$labels, "font")
+    else
+        labfnt <- rep(p$font, length=nr)
+    ## Select only items within xlim, ylim
+    take <- rep(TRUE, nr)
     if (!missing(xlim))
-        sco <- sco[sco[,1] >= xlim[1] & sco[,1] <= xlim[2], , drop = FALSE]
+        take <- take & sco[,1] >= xlim[1] & sco[,1] <= xlim[2]
     if (!missing(ylim))
-        sco <- sco[sco[,2] >= ylim[1] & sco[,2] <= ylim[2], , drop = FALSE]
+        take <- take & sco[,2] >= ylim[1] & sco[,2] <= ylim[2]
+    sco <- sco[take,, drop=FALSE]
     labs <- rownames(sco)
+    pcol <- pcol[take]
+    pbg <- pbg[take]
+    tcol <- tcol[take]
+    pch <- pch[take]
+    diam <- diam[take]
+    labfam <- labfam[take]
+    labsize <- labsize[take]
+    labfnt <- labfnt[take]
     ## Ranges and pretty values for axes
     if (missing(xlim))
         xlim <- range(sco[,1])
@@ -303,17 +409,28 @@
                  text=as.character(tmp[i]), fill = p$col.axis, font=fnt.axis)
     }
     ## Points and labels
-    laboff <- round(p2p * p$ps/2 + diam + 1)
+
+    ## The following 'inherits' works with ordipointlabel, but not
+    ## with zooming
+    if (inherits(x, "orditkplot")) {
+        lsco <- scores(x, "labels")
+        laboff <- rep(0, nrow(lsco))
+        lsco <- lsco[rownames(sco),]
+    } else {
+        lsco <- sco
+        laboff <- round(p2p * p$ps/2 + diam + 1)
+    }
     pola <- tclArray()        # points
     labtext <- tclArray()     # text
     id <- tclArray()          # index
     for (i in 1:nrow(sco)) {
         xy <- usr2xy(sco[i,])
-        item <- tkcreate(can, "oval", xy[1]-diam, xy[2]-diam,
-                         xy[1]+diam,  xy[2]+diam, 
-                         width=1, outline=pcol, fill=pbg)
-        lab <- tkcreate(can, "text", xy[1], xy[2]-laboff, text=labs[i],
-                        fill = p$col, font=fnt)
+        item <- Point(xy[1], xy[2], pch = pch[i], col = pcol[i],
+                      fill = pbg[i], diam = diam[i])
+        xy <- usr2xy(lsco[i,])
+        fnt <- c(labfam[i], labsize[i], saneslant(labfnt[i]))
+        lab <- tkcreate(can, "text", xy[1], xy[2]-laboff[i], text=labs[i],
+                        fill = tcol[i], font=fnt)
         tkaddtag(can, "point", "withtag", item)
         tkaddtag(can, "label", "withtag", lab)
         pola[[lab]] <- item
@@ -326,6 +443,18 @@
 ##############################
     
     ## Plotting and Moving
+    ## Mouse enters a label
+    pEnter <- function() {
+        tkdelete(can, "box")
+        hbox <- tkcreate(can, "rectangle", tkbbox(can, "current"),
+                         outline = "red", fill = "yellow")
+        tkaddtag(can, "box", "withtag", hbox)
+        tkitemraise(can, "current")
+    }
+    ## Mouse leaves a label
+    pLeave <- function() {
+        tkdelete(can, "box")
+    }
     ## Select label
     pDown <- function(x, y) {
         x <- as.numeric(x)
@@ -346,11 +475,15 @@
         y <- as.numeric(y)
         tkmove(can, "selected", x - .lastX, y - .lastY)
         tkdelete(can, "ptr")
+        tkdelete(can, "box")
         .lastX <<- x
         .lastY <<- y
         ## xadj,yadj: adjust for canvas scrolling
         xadj <- as.numeric(tkcanvasx(can, 0))
         yadj <- as.numeric(tkcanvasy(can, 0))
+        hbox <- tkcreate(can, "rectangle", tkbbox(can, "selected"),
+                         outline = "red")
+        tkaddtag(can, "box", "withtag", hbox)
         conn <- tkcreate(can, "line", .lastX + xadj, .lastY+yadj,
                          .pX, .pY, fill="red")
         tkaddtag(can, "ptr", "withtag", conn)
@@ -398,11 +531,17 @@
         tkaddtag(can, "box", "withtag", rect)
     }
     ## Redraw ordiktplot with new xlim and ylim
-    ## FIXME: zooming does not pass "..." arguments
     pZoom <- function() {
-        xlim <- sort(c(x2usr(.pX), x2usr(.lastX)))
-        ylim <- sort(c(y2usr(.pY), y2usr(.lastY)))
-        orditkplot(x, xlim = xlim, ylim = ylim, ...)
+        nxlim <- sort(c(x2usr(.pX), x2usr(.lastX)))
+        nylim <- sort(c(y2usr(.pY), y2usr(.lastY)))
+        xy <- ordDump()
+        ## Move labels closer to points in zoom
+        ## FIXME: Doesn't do a perfect job
+        mul <- abs(diff(nxlim)/diff(xlim))
+        xy$labels <- xy$points + (xy$labels - xy$points)*mul
+        xy$args$xlim <- nxlim
+        xy$args$ylim <- nylim
+        orditkplot(xy)
     }
     ## Dummy location of the mouse
     .lastX <- 0
@@ -411,10 +550,8 @@
     .pY <- 0
     ## Mouse bindings:
     ## Moving a label
-    tkitembind(can, "label", "<Any-Enter>",
-               function() tkitemconfigure(can, "current", fill="red"))
-    tkitembind(can, "label", "<Any-Leave>",
-               function() tkitemconfigure(can, "current", fill=p$col))
+    tkitembind(can, "label", "<Any-Enter>", pEnter)
+    tkitembind(can, "label", "<Any-Leave>", pLeave)
     tkitembind(can, "label", "<1>", pDown)
     tkitembind(can, "label", "<ButtonRelease-1>",
                function() {tkdtag(can, "selected"); tkdelete(can, "ptr")})
